/*******************************************************************
  TCP Socket Stream I/O
  Written by Yasusi Kanada on September 9, 2003
  Updated on October 21, 2003
*******************************************************************/

import java.io.*;
import java.net.*;

/**
 * TCP Socket Stream I/O: A thin protocol layer between TCP and SSIP.
 */
public class TCPSocketIOStream implements SocketIOStream {

    /** TCP socket **/
    ServerSocket server;

    /** Remote message sender's address */
    InetAddress fromAddress;

    /** Input buffer and its size **/
    final int bufferSize = 1500;
    byte[] buffer = new byte[bufferSize];

    /**
     * Constructor
     */
    public TCPSocketIOStream(int localPort) {
        try {
            server = new ServerSocket(localPort);	// Get a TCP socket.
        } catch (Exception x) {
            x.printStackTrace();
        };
    }

    /**
     * Close the stream.
     */
    public void close() {
        if (server != null) {
            try {
                server.close();
            } catch (IOException x) {
            };
        };
    }

    /**
     * Write a message (a TCP packet).
     */
    public boolean write(InetAddress remoteAddress, int remotePort,
                         String message) {
        Socket socket = null;
        boolean result = false;
        try {
            socket = new Socket(remoteAddress, remotePort);
            OutputStream out = socket.getOutputStream();
            out.write(message.getBytes());
            out.flush();
            out.close();
            System.out.println("Message " + message +
                               "  sent to: " + remoteAddress);
            result = true;
        } catch (Exception x) {
            x.printStackTrace();
        } finally {
            if (socket != null) {
                try {
                    socket.close();
                } catch (IOException x) {
                };
            };
        };
        return result;
    }

    /**
     * Read a message (a UDP packet).
     */
    public String read() {
        int length = 0;
        Socket connection = null;
        String result = null;
        try {
            connection = server.accept();
            InputStream in = connection.getInputStream();
            length = in.read(buffer, 0, buffer.length);
            fromAddress =
                ((InetSocketAddress)connection.getRemoteSocketAddress()).
                    getAddress();
            result = new String(buffer, 0, length);
            System.out.println("Message " + result +
                               "  received from: " + fromAddress);
        } catch (Exception x) {
            x.printStackTrace();
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (IOException x) {
                };
            };
        };
        return result;
    }

    /**
     * Get the remote message sender's address.
     * (It is defined only immediately after read() is called.)
     */
    public InetAddress getFromAddress() {
        return fromAddress;
    }
}
