/*******************************************************************
  UDP Socket Stream I/O
  Written by Yasusi Kanada on September 2, 2003
  Updated on September 8 and October 21, 2003
*******************************************************************/

import java.net.*;

/**
 * UDP Socket Stream I/O: A thin protocol layer between UDP and SSIP.
 */
public class UDPSocketIOStream implements SocketIOStream {

    /** UDP socket **/
    DatagramSocket socket;

    /** Remote message sender's address */
    InetAddress fromAddress;

    /** Input buffer and its size **/
    final int bufferSize = 1500;	// LAN MTU must be sufficient.
    byte[] buffer = new byte[bufferSize];

    /**
     * Constructor
     */
    public UDPSocketIOStream(int localPort) {
        try {
            socket = new DatagramSocket(localPort);	// Get a UDP socket.
        } catch (Exception x) {
            x.printStackTrace();
        };
    }

    /**
     * Close the stream.
     */
    public void close() {
        if (socket != null) {
            socket.close();
        };
    }

    /**
     * Write a message (a UDP packet).
     */
    public boolean write(InetAddress remoteAddress, int remotePort,
                         String message) {
        byte[] data = message.getBytes();
        int length = data.length;
        try {
            socket.send(new DatagramPacket(data, 0, length,
                                           remoteAddress, remotePort));
						// Send a UDP packet.
            System.out.println("Message " + message +
                               "  sent to: " + remoteAddress);
        } catch (Exception x) {
            x.printStackTrace();
            return false;
        };
        return true;
    }

    /**
     * Read a message (a UDP packet).
     */
    public String read() {
        DatagramPacket p = new DatagramPacket(buffer, 0, buffer.length);
        try {
            socket.receive(p);	// Receive a UDP packet.
        } catch (Exception x) {
            x.printStackTrace();
            return null;
        };
        fromAddress = p.getAddress();
        String result = new String(buffer, 0, p.getLength());
        System.out.println("Message " + result +
                           "  received from: " + fromAddress);
        return result;
    }

    /**
     * Get the remote message sender's address.
     * (It is defined only immediately after read() is called.)
     */
    public InetAddress getFromAddress() {
        return fromAddress;
    }
}
